
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdlib.h>
#include <string.h>

#define BAUDRATE	B9600

int openMaster(int fd) {
  struct termios oldtio,newtio;
  char device[64];
  char cmd[128];

  if(fd != -1) {
    close(fd);
  }
#ifdef OLD_PTY
  fd = open(DEVICE,O_RDWR);
  if(fd < 0) {
    fprintf(stderr,"*** open %s: %s\n",DEVICE,strerror(errno));
    exit(1);
  }
#else
  fd = getpt();
  if(fd < 0) {
    fprintf(stderr,"*** getpt: %s\n",strerror(errno));
    exit(1);
  }
  if(grantpt(fd) == -1) {
    fprintf(stderr,"*** grantpt: %s\n",strerror(errno));
    exit(1);
  }
  if(unlockpt(fd) == -1) {
    fprintf(stderr,"*** unlockpt: %s\n",strerror(errno));
    exit(1);
  }
  if(ptsname_r(fd,device,sizeof(device)) == -1) {
    fprintf(stderr,"*** ptsname_r: %s\n",strerror(errno));
    exit(1);
  }
  fprintf(stderr,"*** to be used pts: %s\n",device);
  sprintf(cmd,"echo %s > /tmp/rs232dev",device);
  if(system(cmd) != 0) {
    fprintf(stderr,"*** %s failed\n",cmd);
    exit(1);
  }
#endif

#ifndef NO_TERMIOS
  /* set correct baudrate, etc */
  tcgetattr(fd,&oldtio);	/* save current port settings */

  bzero(&newtio,sizeof(newtio));
  newtio.c_cflag = (CS8 | CREAD | CLOCAL | CRTSCTS);
  newtio.c_lflag = 0;
  newtio.c_iflag = IGNPAR;
  newtio.c_oflag = 0;
  cfsetospeed(&newtio,BAUDRATE);
  newtio.c_cc[VTIME] = 0;	/* inter-character timer unused */
  newtio.c_cc[VMIN] = 1;	/* blocking read until 1 char received */
      
  tcflush(fd,TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);
#endif

  return fd;
}

unsigned char packetTemplate[9] = {91,224,0,255,144,255,144,93,93};

int main(int argc,char* argv[]) {
  int fd;
  fd_set readfds;
  int count;

  fd = openMaster(-1);

  FD_ZERO(&readfds);

  while(1) {
    unsigned char buf[512];

    FD_SET(0,&readfds);		/* stdin */
    FD_SET(fd,&readfds);
    count = select(fd+1,&readfds,0,0,0);
    if(count == -1) {
      fprintf(stderr,"*** select: %s\n",strerror(errno));
      return 1;
    }
    if(FD_ISSET(fd,&readfds)) {
      count = read(fd,buf,sizeof(buf));
      if(count == -1) {
	fprintf(stderr,"*** read from rs232: %s (eof in fact,not an error.)\n",
		strerror(errno));
	openMaster(fd);
	continue;
      }
      if(count == 0) {
	fprintf(stderr,"*** rs232 eof\n");
	openMaster(fd);
	continue;
      }
#if 0
      {
	int i;
	
	fprintf(stderr,"got %d bytes:",count);
	for(i = 0; i < count; i++)
	  fprintf(stderr," %d(0x%02x)",buf[i],buf[i]);
	fprintf(stderr,"\n");
      }
#endif
      /* looking if it is a "packet" */
      if(count >= 9 && buf[0] == packetTemplate[0]) {
	int ok = 1;
	int i;
	fprintf(stdout,"maybe a packet:");
	for(i = 0; i < 9; i++) {
	  fprintf(stdout," %d",buf[i]);
	  if(ok && packetTemplate[i] != 255 && buf[i] != packetTemplate[i]) {
	    fprintf(stderr," not a (valid) packet");
	    ok = 0;
	  }
	}
	if(ok) {
	  fprintf(stdout,": good packet: horiz=%d vert=%d\n",buf[3],buf[5]);
	}
	if(count > 9)
	  fwrite(buf+9,1,count-9,stdout);
      } else
	fwrite(buf,1,count,stdout);
    }
    if(FD_ISSET(0,&readfds)) {
      count = read(fd,buf,sizeof(buf));
      if(count == -1) {
	fprintf(stderr,"*** read from stdin: %s\n",strerror(errno));
	return 1;
      }
      if(count == 0) {
	fprintf(stderr,"*** stdin eof\n");
	return 0;
      }
      if(write(fd,buf,count) != count) {
	fprintf(stderr,"*** write to rs232: %s\n",strerror(errno));
	return 1;
      }
    }
  }
  
}
